
from .enzona_api import (
    enzona_api
)

from .enzona_business_payment import (enzona_business_payment, Payments, mProduct, Payload)

from .responses import (response_payments, response_operation_payments, response_return_payments, response_get_refound)

from .error import (EnzonaError)

__title__ = 'enzona_api'
__author__ = 'Josué Carballo Baños'
__license__ = 'GNU v3'
__ver_major__ = 0
__ver_minor__ = 0
__ver_patch__ = 9
__ver_sub__ = ''
__version__ = "%d.%d.%d%s" % (__ver_major__, __ver_minor__,
                              __ver_patch__, __ver_sub__)


from enzona_api.error import EnzonaError


class response_payments():
    def __init__(self, response):
        self.response = response

    def transaction_uuid(self):
        try:
            return self.response['transaction_uuid']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def link_confirm(self):
        try:
            return self.response['links'][0]['href']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def link_complete(self):
        try:
            return self.response['links'][1]['href']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def link_cancel(self):
        try:
            return self.response['links'][2]['href']
        except:
            raise EnzonaError(self.response['fault']['message'])


class response_operation_payments():
    def __init__(self, response):
        self.response = response

    def transaction_uuid(self):
        try:
            return self.response['transaction_uuid']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def status_denom(self):
        try:
            return self.response['status_denom'][0]
        except:
            raise EnzonaError(self.response['fault']['message'])


class response_return_payments():
    def __init__(self, response):
        self.response = response

    def get_payments(self):
        payments = self.response['payments']
        lst_payments = []
        for mpayment in payments:
            lst_payments.append(self.payment(mpayment))
        return lst_payments

    class payment():
        def __init__(self, response): self.response = response

        def invoice_number(self): return self.response['invoice_number']

        def amount_total(self): return self.response['amount']['total']

        def amount_tip(self): return self.response['amount']['details']['tip']

        def amount_discount(self): return self.response['amount']['details'][
            'discount']

        def amount_shipping(self): return self.response['amount']['details'][
            'shipping']

        def amount_tax(self): return self.response['amount']['details']['tax']

        def status_denom(self): return self.response['status_denom']

        def commission(self): return self.response['commission']


class response_refound():
    def __init__(self, response):
        self.response = response

    def transaction_uuid(self):
        try:
            return self.response['transaction_uuid']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def created_at(self):
        try:
            return self.response['created_at']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def amount_total(self):
        try:
            return self.response['amount']['total']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def description(self):
        try:
            return self.response['description']
        except:
            raise EnzonaError(self.response['fault']['message'])

    def refund_fullname(self):
        try:
            return self.response['refund_name'] + ' ' + self.response[
                'refund_lastname']
        except:
            raise EnzonaError(self.response['fault']['message'])


class response_get_refound():
    def __init__(self, response):
        self.response = response

    def get_refunds(self):
        refunds = self.response['refunds']
        lst_refunds = []
        for refund in refunds:
            lst_refunds.append({
                'transaction_uuid': refund['transaction_uuid'],
                'transaction_denom': refund['transaction_denom'],
                'status_denom': refund['status_denom'],
                'transaction_created_at': refund['transaction_created_at'],
                'refund_avatar': refund['refund_avatar'],
                'refund_fullname': refund['name'] + ' ' + refund['lastname'],
                'refunded': refund['amount']['details']['refunded'],
                'total_refunded': refund['amount']['details'][
                    'total_refunded'],
                'total': refund['amount']['total'],
            })
        return lst_refunds



"""
    This library is still in the process of being created. It is not recommended to use it yet in development.

Author: Josué Carballo Baños
License: GNU GPL from the Free Software Foundation v3 and later.
"""

import base64
from io import BytesIO
import requests
import qrcode
import json

from requests import Timeout


class enzona_api():
    token = None
    consumer_key = None
    consumer_secret = None

    def __init__(self, consumer_key, consumer_secret):
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.token = self.get_token()

    def get_token(self):
        """
        :return: Returns an API access token It depends on the input of the public and private keys.
        """
        bs4 = self.get_base64(self.consumer_key + ":" + self.consumer_secret)
        data = {
            "grant_type": "client_credentials",
            'scope': 'enzona_business_payment'
        }
        headers = {
            "Authorization": "Basic " + bs4.split("'")[1]
        }
        response = requests.post("https://api.enzona.net/token", data=data,
                                 headers=headers)
        try:
            if response.status_code != 200:
                return response.reason
            else:
                json_response = response.json()
                self.token = json_response["access_token"]
                return json_response['access_token']

        except ConnectionError as error:
            return {'success': False, 'error': 'Network Error',
                    'error_detail': error}
        except Timeout as error:
            return {'success': False, 'error': 'Network Conection Timeout',
                    'error_detail': error}

    @staticmethod
    def get_base64(text):
        return str(base64.b64encode(bytes(text, "utf-8"))).__str__()



"""
This library is still in the process of being created. It is not recommended
to use it yet in development.

Author: Josué Carballo Baños
License: GNU GPL from the Free Software Foundation v3 and later.
"""

import json
import requests

from enzona_api import enzona_api
from enzona_api.responses import response_payments, \
    response_operation_payments, response_return_payments, response_refound, \
    response_get_refound
from enzona_api.error import EnzonaError


class enzona_business_payment(enzona_api):

    def __init__(self, consumer_key, consumer_secret):
        """
        Initiates entry to the system with the private and secret keys
        :param consumer_key: Private key granted
        :param consumer_secret: Public key granted
        """
        enzona_api.__init__(self, consumer_key, consumer_secret)

    def create_payments(self, payment):
        """
        Allows you to create a payment
        :param payment: Payment object with all payment data
        :return: Return object Json with payment data
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        response = requests.post(
            "https://api.enzona.net/payment/v1.0.0/payments",
            data=json.dumps(payment), headers=headers)
        try:
            if response.status_code != 200:
                return {
                    "error": response.reason,
                    "code": response.status_code,
                    "url": response.url,
                    "headers": response.headers,
                }
            else:
                return response_payments(response.json())
        except EnzonaError as e:
            print(e)

    def cancel_payments(self, transaction_uuid):
        """
        Allows you to cancel a payment
        :param transaction_uuid: Transaction Identifier
        :return: Returns json object with cancellation data
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        response = requests.post(
            "https://api.enzona.net/payment/v1.0.0/payments/{0}/cancel".format(
                transaction_uuid), headers=headers)
        try:
            return response_operation_payments(response.json())
        except EnzonaError as e:
            print(e)

    # Completar un pago
    def complete_payments(self, transaction_uuid):
        """
        Allows you to complete a payment
        :param transaction_uuid: Transaction Identifier
        :return: Returns json object with completed data
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        data = {}
        response = requests.post(
            "https://api.enzona.net/payment/v1.0.0/payments/{0}/"
            "complete".format(transaction_uuid), data=data, headers=headers)
        try:
            return response_operation_payments(response.json())
        except EnzonaError as e:
            print(e)

    def get_payments(self, merchant_uuid, offset=0, limit=10,
                     status_filter=None, start_date_filter="",
                     end_date_filter=""):
        """
        You get a list of payments made
        :param end_date_filter:
        :param start_date_filter:
        :param merchant_uuid: Business Identifier
        :param offset: Where to start showing the data
        :param limit: Amount of data to be displayed
        :param status_filter: Payment status
        :return: Return you get a list of payments made
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        if status_filter is not None:
            status_filter = "&status_filter=" + str(status_filter)
        else:
            status_filter = ""
        response = requests.get(
            "https://api.enzona.net/payment/v1.0.0/payments?" +
            "merchant_uuid=" + merchant_uuid +
            "&limit=" + str(limit) +
            "&offset=" + str(offset) +
            "&order_filter=desc" + status_filter +
            "&start_date_filter=" + start_date_filter +
            "&end_date_filter=" + end_date_filter, headers=headers)
        try:
            return response_return_payments(response.json())
        except EnzonaError as e:
            print(e)

    def payments_refund(self, transaction_uuid, Payload=None):
        """
        :param transaction_uuid: Transaction Identifier
        :param Payload: (Optional) Partial return structure
        :return: Returns the data of the refund transaction
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        if Payload is None:
            payload = {}
        else:
            payload = Payload.get_payload()
        try:
            response = requests.post(
                "https://api.enzona.net/payment/v1.0.0/payments/" +
                transaction_uuid + "/refund",
                data=json.dumps(payload), headers=headers)
            return response_refound(response.json())
        except EnzonaError as e:
            print(e)
            return e

    def get_payments_refund(self, merchant_uuid, offset=0, limit=10,
                            status_filter=None):
        """
        You get a list of returns made
        :param merchant_uuid: Business Identifier
        :param offset: Where to start showing the data
        :param limit: Amount of data to be displayed
        :param status_filter: Payment status
        :return: Return you get a list of returns made
        """
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer {0}".format(self.token)
        }
        if status_filter is not None:
            status_filter = "&status_filter=" + str(status_filter)
        else:
            status_filter = ""
        response = requests.get(
            "https://api.enzona.net/payment/v1.0.0/payments/refunds?"
            "merchant_uuid=" + merchant_uuid + "&limit=" + str(
                limit) + "&offset=" + str(
                offset) + "&order_filter=desc" + status_filter,
            headers=headers)
        try:
            return response_get_refound(response.json())
        except EnzonaError as e:
            print(e)
            return e


class Payload():
    def __init__(self, total, description):
        self.total = total
        self.description = description

    def get_payload(self):
        if len(str(self.total).split(".")[1]) < 2:
            total = str(self.total) + "0"
        else:
            total = self.total
        return {
            "amount": {
                "total": total
            },
            "description": self.description
        }


class mProduct():
    def __init__(self, name, description, quantity, price, tax):
        self.name = name
        self.description = description
        self.quantity = quantity
        self.price = float(price)
        self.tax = float(tax)

    def get_product(self):
        if len(str(self.price).split(".")[1]) < 2:
            price = str(self.price) + "0"
        else:
            price = str(self.price)
        if len(str(self.tax).split(".")[1]) < 2:
            tax = str(self.tax) + "0"
        else:
            tax = str(self.tax)
        return {"name": self.name, "description": self.description,
                "quantity": self.quantity, "price": price, "tax": tax}


class Payments():
    def __init__(self, merchant_uuid, description_payment, currency, shipping,
                 discount, tip, lst_products, merchant_op_id, invoice_number,
                 return_url, cancel_url, terminal_id):

        shipping = float(shipping)
        discount = float(discount)
        tip = float(tip)
        total_price = 0
        for product in lst_products:
            total_price += float(product["quantity"]) * float(product["price"])
        total_tax = 0
        for product in lst_products:
            total_tax += float(product["tax"])

        total_pay = str(round((shipping + total_tax + total_price + (
                    tip - discount)) * 100) / 100)

        if len(str(total_pay).split(".")[1]) < 2:
            total_pay = str(total_pay) + "0"
        else:
            total_pay = str(total_pay)
        if len(str(total_tax).split(".")[1]) < 2:
            total_tax = str(total_tax) + "0"
        else:
            total_tax = str(total_tax)
        if len(str(shipping).split(".")[1]) < 2:
            shipping = str(shipping) + "0"
        else:
            shipping = str(shipping)
        if len(str(discount).split(".")[1]) < 2:
            discount = str(discount) + "0"
        else:
            discount = str(discount)
        if len(str(tip).split(".")[1]) < 2:
            tip = str(tip) + "0"
        else:
            tip = str(tip)

        self.payment_data = {
            "merchant_uuid": merchant_uuid,
            "description": description_payment,
            "currency": currency,
            "amount": {
                "total": str(total_pay),
                "details": {
                    "shipping": shipping,
                    "tax": total_tax,
                    "discount": discount,
                    "tip": tip
                }
            },
            "items": lst_products,
            "merchant_op_id": merchant_op_id,
            "invoice_number": str(invoice_number),
            "return_url": return_url,
            "cancel_url": cancel_url,
            "terminal_id": terminal_id,
            "buyer_identity_code": ""
        }

    def get_payment(self):
        return self.payment_data



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name="enzona-api",
      version="0.1.1",
      description="Enzona's payment platform API access library",
      author="Josué Carballo Baños",
      author_email='josueccb@yandex.com',
      license="GPL 3.0",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/MoonMagiCreation/enzona_api",
      packages=find_packages(),
      install_requires=["requests>=2.23.0", "bs4>=0.0.1", "qrcode>=6.1"],
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
      ],
      python_requires='>=3.5',
)
